﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace TestModelForm
{
    public partial class Form1 : Form
    {
        // setup some drawing tools
        Pen blackPen = new Pen(Color.Black, 2);
        Pen drkGreenPen = new Pen(Color.DarkGreen, 2);
        Pen redPen = new Pen(Color.Red, 3);
        Pen yellowPen = new Pen(Color.Yellow, 3);
        SolidBrush greenBrush = new SolidBrush(Color.LightGreen);
        SolidBrush blackBrush = new SolidBrush(Color.Black);
        SolidBrush whiteBrush = new SolidBrush(Color.White);
        BoardModel engine = new BoardModel();

        // declare variables for UI
        enum Piece { White = 1, Black };
        int boardWidth;
        int sqrSize;
        int boardOffset;
        int currentPlayer = (int)Piece.Black;
        BoardModel model;
        EasyAI computerAI;
        Rectangle validMoveRect;
        Rectangle piecesToFlipRect;
        bool drawValidMoves = false;
        bool drawPiecesToFlip = false;
        string testFileString = "";

        public Form1()
        {
            InitializeComponent();
            // Initialize UI variables
            boardOffset = 50;
            boardWidth = 400;
            sqrSize = boardWidth / 8;

            model = new BoardModel();
            model.newGame();

            // testing area for Easy AI move
            button4.Visible = false;
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            // make sure to set double buffered to true/ add SetStyle lines. 

            // draw Reversi Board
            e.Graphics.FillRectangle(greenBrush, boardOffset, boardOffset, boardWidth, boardWidth);
            for (int i = sqrSize; i <= boardWidth + boardOffset; i += sqrSize)
            {
                e.Graphics.DrawLine(drkGreenPen, boardOffset, i, boardWidth + boardOffset, i);
                e.Graphics.DrawLine(drkGreenPen, i, boardOffset, i, boardWidth + boardOffset);
            }

            // draw board state
            // testing area 
            // Note: to be able to resize board need to compute the 40, 5, etc...
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    int temp = model.getSquareValue(x, y);
                    if (temp  == (int)Piece.Black)
                    {
                        e.Graphics.FillEllipse(blackBrush, (x * sqrSize) + boardOffset + 5,
                            (y * sqrSize) + boardOffset + 5, 40, 40);
                    }
                    else if (temp == (int)Piece.White)
                    {
                        e.Graphics.FillEllipse(whiteBrush, (x * sqrSize) + boardOffset + 5,
                            (y * sqrSize) + boardOffset + 5, 40, 40);
                    }
                }
            }

            // testing area 
            if (drawValidMoves)
            {
                foreach (Square temp in model.legalMovesList)
                {
                    int x = temp.x;
                    int y = temp.y;
                    validMoveRect = new Rectangle((x * sqrSize) + boardOffset + 5, (y * sqrSize) + boardOffset + 5, 40, 40);
                    e.Graphics.DrawRectangle(redPen, validMoveRect);
                }
            }

            if (drawPiecesToFlip)
            {
                foreach (Square temp in model.piecesToFlipList)
                {
                    int x = temp.x;
                    int y = temp.y;
                    piecesToFlipRect = new Rectangle((x * sqrSize) + boardOffset + 5, (y * sqrSize) + boardOffset + 5, 40, 40);
                    e.Graphics.DrawRectangle(yellowPen, piecesToFlipRect);
                }

            }
            
        }

        private void button5_Click(object sender, EventArgs e)
        {
            drawPiecesToFlip = false;
            drawValidMoves = false;
            currentPlayer = (int)Piece.Black;
            textBox2.Text = "currentPlayer = Black";
            this.Invalidate();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            drawPiecesToFlip = false;
            drawValidMoves = false;
            currentPlayer = (int)Piece.White;
            textBox2.Text = "currentPlayer = White";
            this.Invalidate();
        }

        private void Form1_MouseClick(object sender, MouseEventArgs e)
        {
            int clickedX = (e.X) / sqrSize;
            int clickedY = (e.Y) / sqrSize;
            int x = clickedX - 1; // note this fool; dont get yerself a one off error again
            int y = clickedY - 1; // note this fool; dont get yerself a one off error AGAIN
            textBox2.Text = "X: " + x.ToString() + ", Y: " + y.ToString();

            try
            {
                foreach (Square temp in model.legalMovesList)
                {
                    if (x == temp.x && y == temp.y)
                    {
                        textBox2.Text = "Valid move clicked; Pieces to Flip in yellow.";
                        model.loadPiecesToFlip(currentPlayer, x, y);
                        drawPiecesToFlip = true;
                        this.Invalidate();
                    }
                }
            }
            catch (Exception ex) { }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            model.loadLegalMoves(currentPlayer);
            drawValidMoves = true;
            this.Invalidate();

        }

        private void button2_Click(object sender, EventArgs e)
        {
            drawPiecesToFlip = false;
            drawValidMoves = false;
            StreamReader myReader;
            testFileString = textBox1.Text;
            try
            {
                myReader = new StreamReader(testFileString);
                textBox2.Text = "testFile loaded: " + testFileString;

                string temp = myReader.ReadLine();
                int lineCount = Convert.ToInt32(temp);

                model = new BoardModel();
                
                for (int i = 0; i < lineCount; i++)
                {
                    temp = myReader.ReadLine();
                    string[] splitLine = temp.Split(' ');
                    int color, x, y;
                    color = Convert.ToInt32(splitLine[0]);
                    x = Convert.ToInt32(splitLine[1]);
                    y = Convert.ToInt32(splitLine[2]);
                    model.setSquareValue(color, x, y);
                    
                }

                myReader.Close();
                this.Invalidate();
            }
            catch (Exception ex) { textBox2.Text = "Must have valid file name with path."; }

            
            
        }

        private void button3_Click(object sender, EventArgs e)
        {
            model.loadPieceCount();
            textBox3.Text = "Black: " + model.getBlackPieceCount();
            textBox4.Text = "White: " + model.getWhitePieceCount();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            computerAI = new EasyAI((int)Piece.Black, model);
            Square move = computerAI.getComputerMove();
            textBox2.Text = "Computer moves white to: " + move.x + ", " + move.y;
        }

    }
}
